<?php

/**
* class Ability
*
* @version 1.0.3
* @author Baptiste ROUSSEL
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

class Ability implements iterator {
	private static $abilities = array();

	public $id;
	public $name;
	public $abbreviation;
	public $minValue;
	public $maxValue;
	public $gmView;
	public $description;
	private $gmview = false;

	/**
	* Constructor
	*
	* @param  $ (int) Ability's ID
	* @param  $ (Array) Data, default = array()
	*/
	function __construct( $id = 0, $ab = array() )
	{
		if( !is_numeric($id) )
			throw new Exception('Ability\'s ID must be a number.');
		if( $id == 0 )return;
		if ( empty( $data ) ) {
			// Load the ability
			$db = Database::getDatabase();
			$res = $db->prepare( 'SELECT `idability`, `name`, `description`, `abbreviation`, `minvalue`, `maxvalue`, `gmview` FROM `' . TBL_AB . '` WHERE `idability` = :id' );
			$res->bindValue( ':id', $id );
			$res->execute();
			if ( $res->rowCount() == 0 )
				throw new Exception( 'Unknow Ability ' . $id . '.' );

			$ab = $res->fetch();
			$this->id = $id;
			$this->name = $ab['name'];
			$this->abbreviation = $ab['abbreviation'];
			$this->minValue = $ab['minvalue'];
			$this->maxValue = $ab['maxvalue'];
			$this->gmView = (bool)$ab['gmview'];
			$this->description = $ab['description'];
		} else {
			$this->id = $id;
			$this->name = $ab['name'];
			$this->abbreviation = $ab['abbreviation'];
			$this->minValue = $ab['minvalue'];
			$this->maxValue = $ab['maxvalue'];
			$this->gmView = (bool)$ab['gmview'];
			$this->description = $ab['description'];
		}
	}

	/*
	* @param (int) Ability's ID
	* @return (bool)
	*/
	public static function isAbilityLoaded( $id )
	{
		return !empty( self::$abilities[$id] );
	}

	/*
	* Load all the abilities from the database
	* @retun (int) Number of ability loaded
	*/
	public static function loadAll()
	{
		$db = Database::getDatabase();
		$res = $db->query( 'SELECT `idability` AS `id`, `name`, `description`, `abbreviation`, `minvalue`, `maxvalue`, `gmview` FROM `' . TBL_AB . '` ORDER BY `idability` ASC' );
		if ( $res->rowCount() == 0 )
			return 0;

		$nbLoaded = 0;
		foreach( $res as $ab ) {
			// Verify that the ability is not already loaded
			if ( self::isAbilityLoaded( $ab['id'] ) === false ) {
				// Store the ability
				self::$abilities[$ab['id']] = new Ability( $ab['id'], $ab );
				$nbLoaded++;
			}
		}

		return $nbLoaded;
	}

	/*
	* @param (int) Ability's ID
	* @return (Ability)
	*/
	public static function getAbility( $id )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Ability\'s ID must be a number.' );
		if ( self::isAbilityLoaded( $id ) )
			return self::$abilities[$id];
		else {
			return new Ability( $id );
		}
	}

	/*
	* Return the first ability loaded
	* @return (Skill) or null if no ability loaded
	*/
	public static function getFirst(){
		$key = array_keys(self::$abilities);
		if( sizeof($key) == 0 )
			return null;
		return self::$abilities[$key[0]];
	}

	/*
	* Getter
	* @param (String) Variable name
	* @return (mixed) Value
	*/
	public function __get( $name )
	{
		if ( isSet( $this->$$name ) && $name != 'abilities' )
			return $this->$$name;
		return null;
	}

	public static function &getName( $id )
	{
		if ( self::isAbilityLoaded( $id ) === false )
			self::$abilities[$id] = new Ability( $id );
		return self::$abilities[$id]->name;
	}

	public static function &getAbbreviation( $id )
	{
		if ( self::isAbilityLoaded( $id ) === false )
			self::$abilities[$id] = new Ability( $id );
		return self::$abilities[$id]->abbreviation;
	}

	public static function &getMinValue( $id )
	{
		if ( self::isAbilityLoaded( $id ) === false )
			self::$abilities[$id] = new Ability( $id );
		return self::$abilities[$id]->minValue;
	}

	public static function &getMaxValue( $id )
	{
		if ( self::isAbilityLoaded( $id ) === false )
			self::$abilities[$id] = new Ability( $id );
		return self::$abilities[$id]->maxValue;
	}

	public static function isGmView( $id )
	{
		if ( self::isAbilityLoaded( $id ) === false )
			self::$abilities[$id] = new Ability( $id );
		return self::$abilities[$id]->gmView;
	}

	public static function &getDescription( $id )
	{
		if ( self::isAbilityLoaded( $id ) === false )
			self::$abilities[$id] = new Ability( $id );
		return self::$abilities[$id]->description;
	}

	public function save(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		// Look if it is an update or a creation
		$update = ($this->id != 0) ? true : false;
		// table : ability
		if( $update === true ){
			$res = $db->prepare('UPDATE `' . TBL_AB . '` SET `name` = :name, `abbreviation` = :abbreviation, `minvalue` = :minvalue, `maxvalue` = :maxvalue, `description` = :description, `gmview` = :gm_view WHERE `idability` = :id');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':abbreviation', $this->abbreviation, PDO::PARAM_STR);
			$res->bindValue(':minvalue', $this->minValue, PDO::PARAM_INT);
			$res->bindValue(':maxvalue', $this->maxValue, PDO::PARAM_INT);
			$res->bindValue(':description', $this->description, PDO::PARAM_STR);
			$res->bindValue(':gm_view', (int)$this->gmView, PDO::PARAM_INT);
			$res->bindValue(':id', $this->id, PDO::PARAM_INT);
			$res->execute();
			// update sheets with new min and max
			$datas = $db->query('SELECT `' . TBL_D . '`.`iddata`,`rolledvaluetype`,`rolledvalueclasse`,`bonusvalue` FROM `' . TBL_D . '` LEFT JOIN `' . TBL_D_AB . '` ON (`' . TBL_D . '`.`iddata`=`' . TBL_D_AB . '`.`iddata` AND `' . TBL_D_AB . '`.`idability`=' . $db->quote($this->id) . ')');
			$res = $db->prepare('UPDATE `' . TBL_D_AB . '` SET `bonusvalue` = :bonusvalue WHERE `idability` = :idability AND `iddata` = :id');
			$res->bindValue(':idability',$this->id,PDO::PARAM_INT);
			foreach($datas as $data){
				if( $data['rolledvaluetype'] + $data['rolledvalueclasse'] + $data['bonusvalue'] < $this->minValue ){
					$bonus = $this->minValue - $data['rolledvaluetype'] - $data['rolledvalueclasse'] - $data['bonusvalue'];
					$res->bindValue(':bonusvalue', $bonus, PDO::PARAM_INT);
					$res->bindValue(':id',$data['iddata'],PDO::PARAM_INT);
					$res->execute();
				}elseif( $data['rolledvaluetype'] + $data['rolledvalueclasse'] + $data['bonusvalue'] > $this->maxValue ){
					$bonus = $data['bonusvalue'] - ($data['rolledvaluetype'] + $data['rolledvalueclasse'] + $data['bonusvalue'] - $this->maxValue);
					$res->bindValue(':bonusvalue', $bonus, PDO::PARAM_INT);
					$res->bindValue(':id',$data['iddata'],PDO::PARAM_INT);
					$res->execute();
				}
			}
		}else{
			$res = $db->prepare('INSERT into `' . TBL_AB . '`(`name`,`abbreviation`,`minvalue`,`maxvalue`,`description`,`gmview`) VALUES (:name, :abbreviation, :minvalue, :maxvalue, :description, :gm_view)');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':abbreviation', $this->abbreviation, PDO::PARAM_STR);
			$res->bindValue(':minvalue', $this->minValue, PDO::PARAM_INT);
			$res->bindValue(':maxvalue', $this->maxValue, PDO::PARAM_INT);
			$res->bindValue(':description', $this->description, PDO::PARAM_STR);
			$res->bindValue(':gm_view', (int)$this->gmView, PDO::PARAM_INT);
			$res->execute();
			$this->id = $db->lastInsertId();
			// update sheets
			$datas = $db->query('SELECT `iddata` FROM `' . TBL_D . '`');
			$res = $db->prepare('INSERT into `' . TBL_D_AB . '`(`iddata`,`idability`,`rolledvalueclasse`,`rolledvaluetype`,`bonusvalue`) VALUES(:id,:idability,:rolledvalueclasse,:rolledvaluetype,:bonusvalue)');
			$res->bindValue(':idability',$this->id,PDO::PARAM_INT);
			$res->bindValue(':rolledvaluetype',0,PDO::PARAM_STR);
			$res->bindValue(':rolledvalueclasse',0,PDO::PARAM_STR);
			$res->bindValue(':bonusvalue',$this->minValue,PDO::PARAM_STR);
			foreach($datas as $data){
				$res->bindValue(':bonusvalue',$this->minValue,PDO::PARAM_STR);
				$res->bindValue(':id',$data['iddata'],PDO::PARAM_INT);
				$res->execute();
			}
		}
		$db->commit();
		return true;
	}

	public function delete(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		$db->query('DELETE FROM `' . TBL_D_AB . '` WHERE `idability` = ' . $db->quote($this->id));
		$db->query('DELETE FROM `' . TBL_AB . '` WHERE `idability` = ' . $db->quote($this->id));
		$db->commit();
		return true;
	}

	public function rewind() {
		reset(self::$abilities);
	}

	public function current() {
		return current(self::$abilities);
	}

	public function key() {
		return key(self::$abilities);
	}

	public function next() {
		next(self::$abilities);
	}

	public function valid() {
		return (current(self::$abilities) != false);
	}
}

?>