<?php

/**
* Class Data
*
* @version 1.0.3
* @author Baptiste ROUSSEL
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

class Data implements Iterator {
	private static $datas = array();

	public $id = 0;
	public $name = '';
	public $image = '';
	public $type = null;
	public $classe = null;
	public $abilities = array();
	public $fields = array();
	public $attributes = array();
	public $skills = array();
	public $items = array();
	public $created = '';
	public $modified = '';

	const PARTIAL = 0;
	const FULL = 1;

	private $load = Data::PARTIAL;

	/**
	* Constructor
	*/
	function __construct( $id = 0, $load = Data::PARTIAL )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Data\'s ID must be a number.' );
		if( $id === 0 )
			return;
		// Load the data from the database
		$db = Database::getDatabase();
		$res = $db->prepare( 'SELECT `name`, `image`, `idtype`, `idclasse`, `created`, `modified` FROM `' . TBL_D . '` WHERE `iddata` = :id' );
		$res->bindValue( ':id', $id );
		$res->execute();
		if ( $res->rowCount() == 1 ) {
			$c = $res->fetch();
			$this->id = $id;
			$this->name = $c['name'];
			$this->image = $c['image'];
			$this->type = Type::getType( $c['idtype'] );
			$this->classe = Classe::getClasse( $c['idclasse'] );
			$this->created = $c['created'];
			$this->modified = $c['modified'];
			$res->closeCursor();
			if( $load == Data::FULL ){
				// Full load is required
				$this->fullLoad();
			}
			self::$datas[$this->id] = $this;
		} else
			throw new Exception( 'Unknow Data ' . $id . '.' );
	}

	/*
	* Load Abilities, Attributes, BigFields, Skills, Items
	*/
	private function fullLoad(){
		$db = Database::getDatabase();
		// Get abilities
		$res = $db->prepare( 'SELECT `idability` AS `id`, `rolledvalueclasse`, `rolledvaluetype`, `bonusvalue` FROM `' . TBL_D_AB . '` WHERE `iddata` = :id' );
		$res->bindValue( ':id', $this->id );
		$res->execute();
		if( $res->rowCount() > 0 ){
			foreach($res as $abilitie){
				$typeAb = $this->type->getAbility($abilitie['id']);
				$typeBonus = ($typeAb !== false)? $typeAb['bonusValue'] : 0;
				$classeAb = $this->classe->getAbility($abilitie['id']);
				$classeBonus = ($classeAb !== false)? $classeAb['bonusValue'] : 0;
				$this->abilities[$abilitie['id']] = array(
					'name' => Ability::getName($abilitie['id']),
					'gmview' => Ability::isGmView($abilitie['id']),
					'type' => array(
								'rolled' => $abilitie['rolledvaluetype'],
								'bonus' => $typeBonus
							),
					'classe' => array(
								'rolled' => $abilitie['rolledvalueclasse'],
								'bonus' => $classeBonus
							),
					'bonus' => $abilitie['bonusvalue']
				);
			}
			$res->closeCursor();
		}
		// Get attributes
		$res = $db->prepare( 'SELECT `' . TBL_ATT . '`.`idattribute` AS `id` , `' . TBL_ATT . '`.`name`, `' . TBL_ATT . '`.`description`, `' . TBL_ATT . '`.`gmview` , `' . TBL_D_ATT . '`.`value` FROM `' . TBL_ATT . '` LEFT JOIN `' . TBL_D_ATT . '` ON `' . TBL_ATT . '`.`idattribute` = `' . TBL_D_ATT . '`.`idattribute` AND `' . TBL_D_ATT . '`.`iddata` = :id' );
		$res->bindValue( ':id', $this->id );
		$res->execute();
		if( $res->rowCount() > 0 ){
			foreach($res as $att){
				$this->attributes[$att['id']] = array(
					'name' => $att['name'],
					'value' => $att['value'],
					'description' => $att['description'],
					'gmview' => (bool)$att['gmview']
				);
			}
			$res->closeCursor();
		}
		// Get bigfields
		$res = $db->prepare( 'SELECT `' . TBL_BIG . '`.`idbigfield` AS `id` , `' . TBL_BIG . '`.`name` , `' . TBL_BIG . '`.`gmview`, `' . TBL_D_BIG . '`.`value` FROM `' . TBL_BIG . '` LEFT JOIN `' . TBL_D_BIG . '` ON `' . TBL_BIG . '`.`idbigfield` = `' . TBL_D_BIG . '`.`idbigfield` AND `' . TBL_D_BIG . '`.`iddata` = :id' );
		$res->bindValue( ':id', $this->id );
		$res->execute();
		if( $res->rowCount() > 0 ){
			foreach($res as $field){
				$this->fields[$field['id']] = array(
					'name' => $field['name'],
					'value' => $field['value'],
					'gmview' => (bool)$field['gmview']
				);
			}
			$res->closeCursor();
		}
		// Get skills
		$res = $db->prepare( 'SELECT `idskill` AS `id`, `value` FROM `' . TBL_D_S . '` WHERE `iddata` = :id' );
		$res->bindValue( ':id', $this->id );
		$res->execute();
		if( $res->rowCount() > 0 ){
			foreach($res as $skill){
				$this->skills[$skill['id']] = Skill::getSkill($skill['id']);
			}
			$res->closeCursor();
		}
		// Get Items
		$res = $db->prepare( 'SELECT `' . TBL_I . '`.`iditem` AS `id`, `quantity`, `item_category`.`name` AS `category` FROM `' . TBL_D_I . '` INNER JOIN `' . TBL_I . '` ON `' . TBL_D_I . '`.`iditem` = `' . TBL_I . '`.`iditem` INNER JOIN `' . TBL_I_CAT . '` ON `' . TBL_I . '`.`iditemcategory` = `' . TBL_I_CAT . '`.`iditemcategory` WHERE `iddata` = :id ORDER BY `' . TBL_I_CAT . '`.`iditemcategory`' );
		$res->bindValue( ':id', $this->id );
		$res->execute();
		if( $res->rowCount() > 0 ){
			foreach($res as $item){
				$this->items[] = array(
					'info' => Item::getITem($item['id']),
					'quantity' => $item['quantity'],
					'category' => $item['category']
				);
			}
			$res->closeCursor();
		}
		$this->load = Data::FULL;
	}

	public function isFullLoaded(){
		return ($this->load === Data::FULL)? true : false;
	}

	/*
	* Getter
	* @param (String) Variable's name
	* @return (mixed) Value
	*/
	public function __get( $name )
	{
		if ( isSet( $this->$name ) )
			return $this->$name;
		return null;
	}

	/*
	* Setter
	* @param (String) Variable's name
	* @param (mixed) Value
	*/
	public function __set($name, $value){
		if( isSet( $this->$name) )
			$this->$name = $value;
	}

	public function hasSkill($id){
		return isSet($this->skills[$id]);
	}

	public function getAbilities(){
		$return = array();
		foreach($this->abilities as $id => $ab){
			$typeAb = $this->type->getAbility($id);
			$typeBonus = ($typeAb !== false)? $typeAb['bonusValue'] : 0;
			$classeAb = $this->classe->getAbility($id);
			$classeBonus = ($classeAb !== false)? $classeAb['bonusValue'] : 0;
			$return[$id] = array(
				'type' => array(
							'rolled' => $ab['rolledValueType'],
							'bonus' => $raceBonus
						),
				'classe' => array(
							'rolled' => $ab['rolledValueClasse'],
							'bonus' => $classeBonus
						),
				'bonus' => $ab['bonusvalue']

			);
		}
		return $return;
	}

	/*
	* Save the Data's modifications
	* @return (Boolean)
	*/
	public function save(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		// Look if it is an update or a creation
		$update = ($this->id != 0) ? true : false;
		// Verify fields
		/*if( empty($this->name) || empty($this->race) || empty($this->classe) )
			return false;*/
		// table : data
		if( $update === true ){
			$res = $db->prepare('UPDATE `' . TBL_D . '` SET `name` = :name, `idtype` = :idtype, `idclasse` = :idclasse, `image` = :image, `modified` = NOW() WHERE `iddata` = :id');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':idtype', $this->type->id, PDO::PARAM_INT);
			$res->bindValue(':idclasse', $this->classe->id, PDO::PARAM_INT);
			$res->bindValue(':image', $this->image, PDO::PARAM_STR);
			$res->bindValue(':id', $this->id, PDO::PARAM_INT);
			$res->execute();
		}else{
			$res = $db->prepare('INSERT into `' . TBL_D . '`(`name`,`idtype`,`idclasse`,`image`,`modified`,`created`) VALUES (:name, :idtype, :idclasse, :image, NOW(), NOW())');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':idtype', $this->type->id, PDO::PARAM_INT);
			$res->bindValue(':idclasse', $this->classe->id, PDO::PARAM_INT);
			$res->bindValue(':image', $this->image, PDO::PARAM_STR);
			$res->execute();
			$this->id = $db->lastInsertId();
		}
		// table : data_ability
		$db->query('DELETE FROM `' . TBL_D_AB . '` WHERE `iddata` = ' . $db->quote($this->id));
		$res = $db->prepare('INSERT into `' . TBL_D_AB . '`(`rolledvalueclasse`,`rolledvaluetype`,`bonusvalue`,`iddata`,`idability`) VALUES(:rolledclasse, :rolledtype, :bonus, :id, :idability)');
		foreach($this->abilities as $id => $ab){
			$res->bindValue(':rolledtype',$ab['rolledValueType'],PDO::PARAM_INT);
			$res->bindValue(':rolledclasse',$ab['rolledValueClasse'],PDO::PARAM_INT);
			if( !isSet($ab['bonusValue']) )
				$ab['bonusValue'] = 0;
			$res->bindValue(':bonus',0,PDO::PARAM_INT);
			$res->bindValue(':id',$this->id,PDO::PARAM_INT);
			$res->bindValue(':idability',$id,PDO::PARAM_INT);
			$res->execute();
		}
		// table : data_skill
		$del = $db->prepare('DELETE FROM `' . TBL_D_S . '` WHERE `iddata` = :id');
		$del->bindValue(':id',$this->id,PDO::PARAM_INT);
		$del->execute();
		$res = $db->prepare('INSERT into `' . TBL_D_S . '`(`idskill`,`iddata`,`value`) VALUES(:idskill,:id,:value)');
		$res->bindValue(':id',$this->id,PDO::PARAM_INT);
		foreach($this->skills as $id => $skill){
			$res->bindValue(':idskill',$id,PDO::PARAM_INT);
			$res->bindValue(':value',$skill,PDO::PARAM_STR);
			$res->execute();
		}
		// table : data_item
		$del = $db->prepare('DELETE FROM `' . TBL_D_I . '` WHERE `iddata` = :id');
		$del->bindValue(':id',$this->id,PDO::PARAM_INT);
		$del->execute();
		$res = $db->prepare('INSERT into `' . TBL_D_I . '`(`iditem`,`iddata`,`quantity`) VALUES(:iditem,:id,:quantity)');
		$res->bindValue(':id',$this->id,PDO::PARAM_INT);
		foreach($this->items as $id => $item){
			$res->bindValue(':iditem',$id,PDO::PARAM_INT);
			$res->bindValue(':quantity',$item['quantity'],PDO::PARAM_INT);
			$res->execute();
		}
		// table : data_bigfield
		$del = $db->prepare('DELETE FROM `' . TBL_D_BIG . '` WHERE `iddata` = :id AND `idbigfield` = :idfield');
		$res = $db->prepare('INSERT into `' . TBL_D_BIG . '`(`iddata`,`idbigfield`,`value`) VALUES(:id, :idfield, :value)');
		$res->bindValue(':id',$this->id);
		$del->bindValue(':id',$this->id);
		foreach($this->fields as $id => $field){
			if( isSet($field['value']) && $field['value'] != null ){
				$del->bindValue(':idfield',$id);
				$del->execute();
				$res->bindValue(':idfield',$id);
				$res->bindValue(':value',$field['value']);
				$res->execute();
			}
		}
		// table : data_attribute
		$del = $db->prepare('DELETE FROM `' . TBL_D_ATT . '` WHERE `iddata` = :id AND `idattribute` = :idfield');
		$res = $db->prepare('INSERT into `' . TBL_D_ATT . '`(`iddata`,`idattribute`,`value`) VALUES(:id, :idfield, :value)');
		$res->bindValue(':id',$this->id);
		$del->bindValue(':id',$this->id);
		foreach($this->attributes as $id => $field){
			if( isSet($field['value']) && $field['value'] != null ){
				$del->bindValue(':idfield',$id);
				$del->execute();
				$res->bindValue(':idfield',$id);
				$res->bindValue(':value',$field['value']);
				$res->execute();
			}
		}

		$db->commit();
		return true;
	}

	/*
	* Delete all about the data in the database
	* @return (Boolean)
	*/
	public function delete(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		$db->query('DELETE FROM `' . TBL_D_ATT . '` WHERE `iddata` = ' . $db->quote($this->id));
		$db->query('DELETE FROM `' . TBL_D_BIG . '` WHERE `iddata` = ' . $db->quote($this->id));
		$db->query('DELETE FROM `' . TBL_D_I . '` WHERE `iddata` = ' . $db->quote($this->id));
		$db->query('DELETE FROM `' . TBL_D_S . '` WHERE `iddata` = ' . $db->quote($this->id));
		$db->query('DELETE FROM `' . TBL_D_AB . '` WHERE `iddata` = ' . $db->quote($this->id));
		$db->query('DELETE FROM `' . TBL_D . '` WHERE `iddata` = ' . $db->quote($this->id));
		$db->commit();
		return true;
	}

	public function rewind() {
		reset(self::$datas);
	}

	public function current() {
		return current(self::$datas);
	}

	public function key() {
		return key(self::$datas);
	}

	public function next() {
		next(self::$datas);
	}

	public function valid() {
		return (current(self::$datas) != false);
	}
}

?>