<?php

/**
* class Database
*
* @version 1.0.3
* @author Baptiste ROUSSEL
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

// Table name
if( !defined('TBL') ){
	define('TBL',true);
	define('TBL_AB','ability');
	define('TBL_ACC','access');
	define('TBL_ATT','attribute');
	define('TBL_BIG','bigfield');
	define('TBL_CLA','classe');
	define('TBL_CLA_AB','classe_ability');
	define('TBL_D','data');
	define('TBL_D_AB','data_ability');
	define('TBL_D_ATT','data_attribute');
	define('TBL_D_BIG','data_bigfield');
	define('TBL_D_I','data_item');
	define('TBL_D_S','data_skill');
	define('TBL_GM_ACC','gm_access');
	define('TBL_GM_ACT','gm_account');
	define('TBL_GM_LOG','gm_log');
	define('TBL_I','item');
	define('TBL_I_CAT','item_category');
	define('TBL_S','skill');
	define('TBL_T','type');
	define('TBL_T_AB','type_ability');
}

class Database extends PDO {
	private static $instance = null;
	private $transaction = false;

	/*
	* Constructor
	*/
	function __construct( $dsn, $login, $password, $options = array() )
	{
		if ( self::$instance === null ) {
			parent::__construct( $dsn, $login, $password, $options );
			$this->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
			self::$instance = $this;
		}
	}

	/*
	* Getter
	* @return (Database)
	* @throw (Exception) if no database connexion is open
	*/
	public static function getDatabase()
	{
		if ( self::$instance !== null )
			return self::$instance;
		else
			throw new Exception( 'No database connexion opened.' );
	}

	public function beginTransaction(){
		$this->transaction = true;
		parent::beginTransaction();
	}

	public function commit(){
		if( $this->transaction === true )
			parent::commit();
	}

	public function rollback(){
		if( $this->transaction === true )
			parent::rollback();
	}
}

?>