<?php

/**
* class Dice
*
* Dice format : [numberOfDice]D[numberOfFace]
* Example : 3D12
* N.B :
* 	 D8 => 1D8
* 	 2D => 2D6
* 	 D => 1D6
*
* @version 1.0.0
* @author Baptiste ROUSSEL
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

class Dice {
	private $dice = '';
	private $numberOfDice = null;
	private $numberOfFace = null;
	private $values = array();

	/*
	* Constructor
	* @param (String) Dice, default = 'D6'
	* @param (Array) Values, default = array()
	*/
	function __construct( $dice = 'D6', $values = array() )
	{
		if( $dice != 0 ){
			$tmp = explode( 'D', $dice );
			if ( sizeof( $tmp ) == 2 ) {
				if ( !empty( $tmp[0] ) )
					$this->numberOfDice = intval( $tmp[0] );
				else
					$this->numberOfDice = 1;

				if ( !empty( $tmp[1] ) )
					$this->numberOfFace = intval( $tmp[1] );
				else
					$this->numberOfFace = 6;
			}
			$this->dice = $this->numberOfDice.'D'.$this->numberOfFace;
		}
		else
			$this->dice = '-';
		$this->values = $values;
	}

	/*
	* Return the value, roll the dice(s) if no value memorized.
	* @return (int) Value
	*/
	public function value()
	{
		if ( $this->numberOfDice !== null && $this->numberOfFace !== null ) {
			if ( sizeof( $this->values ) == 0 && $this->numberOfFace > 0 && $this->numberOfDice > 0 ) {
				for( $i = 0; $i < $this->numberOfDice; $i++ )
					$this->values[] = mt_rand( 1, 100 ) % $this->numberOfFace + 1;
			}
		}
		return array_sum( $this->values );
	}

	/*
	* Roll the dice(s)
	* @param (bool) if true, the dice(s) will not be rolled if there is already a result. default = false
	* @return (bool)
	*/
	public function roll( $onlyOnce = false )
	{
		if ( sizeof( $this->values ) !== 0 && $onlyOnce === true )
			return false;
		if ( $this->numberOfFace <= 0 || $this->numberOfDice <= 0 )
			return false;
		for( $i = 0; $i < $this->numberOfDice; $i++ )
			$this->values[] = mt_rand( 1, 100 ) % $this->numberOfFace + 1;
		return true;
	}

	/*
	* @return (String) Dice(s)
	*/
	public function getDice(){
		return $this->dice;
	}

	/*
	* Return the value, roll the dice(s) if no value memorized.
	* @return (String) Value
	*/
	public function __toString(){
		return (String)$this->value();
	}
}

?>