<?php

/**
 * class Item
 *
 * @version 1.0.3
 * @author Baptiste ROUSSEL
 * @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

class Item implements Iterator{
	private static $items = array();
	private static $itemsByCat = array();
	private static $position = 1;
	const FETCH_BYID = 0x1;
	const FETCH_BYCAT = 0x2;
	private static $fetchMode = self::FETCH_BYID;

	public $id = 0;
	public $name = '';
	public $description = '';
	public $icon = '';
	public $category = '';
	public $categoryid = 0;
	public $gmview = false;

	/*
	* Constructor
	* @param (int) Item's ID
	* @param (array) Data, default = array()
	*/
	function __construct( $id = 0, $data = array() ){
		if( !is_numeric($id) )
			throw new Exception('Item\'s ID must be a number.');
		if( $id === 0 )return;
		if( self::isItemLoaded($id) === false ){
			if ( empty( $data ) ) {
				// Load the ability
				$db = Database::getDatabase();
				$res = $db->prepare( 'SELECT `' . TBL_I . '`.`name`, `description`, `icon`, `gmview` , `' . TBL_I . '`.`iditemcategory`, `' . TBL_I_CAT . '`.`name` AS `category` FROM `' . TBL_I . '` INNER JOIN `' . TBL_I_CAT . '` ON `' . TBL_I . '`.`iditemcategory` = `' . TBL_I_CAT . '`.`iditemcategory` WHERE `iditem` = :id' );
				$res->bindValue( ':id', $id );
				$res->execute();
				if ( $res->rowCount() == 0 )
					throw new Exception( 'Unknow Item ' . $id . '.' );

				$item = $res->fetch();
				$this->id = $id;
				$this->name = $item['name'];
				$this->icon = $item['icon'];
				$this->description = $item['description'];
				$this->category = $item['category'];
				$this->categoryid = $item['iditemcategory'];
				$this->gmview = (bool)$item['gmview'];
			} else {
				$this->id = $id;
				$this->name = ( isSet($data['name']) ) ? $data['name'] : '';
				$this->icon = ( isSet($data['icon']) ) ? $data['icon'] : '';
				$this->description = ( isSet($data['description']) ) ? $data['description'] : '';
				$this->category = ( isSet($data['category']) ) ? $data['category'] : '';
				$this->categoryid = ( isSet($data['categoryid']) ) ? $data['categoryid'] : '';
				$this->gmview = ( isSet($data['gmview']) ) ? (bool)$data['gmview'] : false;
			}
			self::$items[$id] = &$this;
		} else {
			// Copy of the Item already loaded
			$item = self::$items[$id];
			$this->id = $id;
			$this->name = $item->name;
			$this->description = $item->description;
			$this->icon = $item->icon;
			$this->category = $item->category;
			$this->categoryid = $item->categoryid;
			$this->gmview = $item->gmview;
		}
	}

	/*
	* Getter
	* @param (String) Variable name
	* @return (mixed) Value
	*/
	public function __get( $name ){
		if( isSet($this->$name) && $name != 'items' )
			return $this->$name;
		return null;
	}

	public static function isItemLoaded( $id ){
		return (isSet(self::$items[$id]));
	}

	public static function getItem( $id ){
		if( self::isItemLoaded($id) )
			return self::$items[$id];
		else
			return new Item($id);
	}

	/*
	* Return the first Item loaded
	* @return (Skill) or null if no item loaded
	*/
	public static function getFirst(){
		$key = array_keys(self::$items);
		if( sizeof($key) == 0 )
			return null;
		return self::$items[$key[0]];
	}

	/*
	   * Load an item from the database
	   * @param (int) Item's ID
	   * @return (bool)
	*/
	private static function loadItem( $id )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Item\'s ID must be a number.' );
		if ( !self::isItemLoaded( $id ) )
			new Item( $id );
		return true;
	}

	public static function loadAll(){
		$db = Database::getDatabase();
		$res = $db->query('SELECT `iditem` FROM `' . TBL_I . '` ORDER BY `iditemcategory`');
		if( $res->rowCount() > 0 ){
			foreach($res as $r){
				if( self::loadItem($r['iditem']) === true ){
					self::$itemsByCat[] = $r['iditem'];
				}
			}
		}
	}

	public function save(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		// Look if it is an update or a creation
		$update = ($this->id != 0) ? true : false;
		// table : item
		if( $update === true ){
			$res = $db->prepare('UPDATE `' . TBL_I . '` SET `name` = :name, `iditemcategory` = :iditemcategory, `description` = :description, `icon` = :icon, `gmview` = :gm_view WHERE `iditem` = :id');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':iditemcategory', $this->categoryid, PDO::PARAM_INT);
			$res->bindValue(':description', $this->description, PDO::PARAM_STR);
			$res->bindValue(':icon', $this->icon, PDO::PARAM_STR);
			$res->bindValue(':gm_view', (int)$this->gmView, PDO::PARAM_INT);
			$res->bindValue(':id', $this->id, PDO::PARAM_INT);
			$res->execute();
		}else{
			$res = $db->prepare('INSERT into `' . TBL_I . '`(`name`,`iditemcategory`,`description`,`icon`,`gmview`) VALUES (:name, :iditemcategory, :description, :icon, :gm_view)');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':iditemcategory', $this->categoryid, PDO::PARAM_INT);
			$res->bindValue(':description', $this->description, PDO::PARAM_STR);
			$res->bindValue(':icon', $this->icon, PDO::PARAM_STR);
			$res->bindValue(':gm_view', (int)$this->gmView, PDO::PARAM_INT);
			$res->execute();
			$this->id = $db->lastInsertId();
		}
		$db->commit();
		return true;
	}

	/*
	* Delete item from the database 
	*/
	public function delete(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		$db->query('DELETE FROM `' . TBL_D_I . '` WHERE `iditem` = ' . $db->quote($this->id));
		$db->query('DELETE FROM `' . TBL_I . '` WHERE `iditem` = ' . $db->quote($this->id));
		$db->commit();
		return true;
	}

	public static function setFetchMode($mode){
		if( $mode == self::FETCH_BYCAT || $mode == self::FETCH_BYID ){
			self::$fetchMode = $mode;
			return true;
		}
		return false;
	}

	public static function getFetchMode(){
		return self::$fetchMode;
	}

	public function rewind() {
		if( self::$fetchMode == self::FETCH_BYID )
			self::$position = 1;
		else
			self::$position = 0;
	}

	public function current() {
		if( self::$fetchMode == self::FETCH_BYID )
			return self::$items[self::$position];
		else
			return self::$items[self::$itemsByCat[self::$position]];
	}

	public function key() {
		return self::$position;
	}

	public function next() {
		++self::$position;
	}

	public function valid() {
		if( self::$fetchMode == self::FETCH_BYID )
			return self::isItemLoaded(self::$position);
		else
			return isset(self::$itemsByCat[self::$position]);
	}
}

?>