<?php

/**
* class Skill
*
* @version 1.0.3
* @author Baptiste ROUSSEL
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/

class Skill implements Iterator{
	private static $skills = array();

	public $id = 0;
	public $name = '';
	public $description = '';
	public $icon = '';
	public $gmview = false;

	public function __construct( $id = 0 )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Skill\'s ID must be a number.' );
		if( $id == 0 )return;
		if ( $this->isSkillLoaded( $id ) === false ) {
			// Load the Skill from the database
			$this->id = $id;
			$db = Database::getDatabase();
			$res = $db->prepare( 'SELECT `name`, `description`, `icon`, `gmview` FROM `' . TBL_S . '` WHERE `idskill` = :id' );
			$res->bindValue( ':id', $id );
			$res->execute();
			if ( $res->rowCount() == 1 ) {
				$f = $res->fetch();
				$this->name = $f['name'];
				$this->description = $f['description'];
				$this->icon = $f['icon'];
				$this->gmview = (bool)$f['gmview'];
				$res->closeCursor();
			} else
				throw new Exception( 'Unknow Skill ' . $id . '.' );

			self::$skills[$id] = $this;
		} else {
			// Copy of the Classe already loaded
			$skill = self::$skills[$id];
			$this->id = $id;
			$this->name = $skill->name;
			$this->description = $skill->description;
			$this->icon = $skill->icon;
			$this->gmview = $skill->gmview;
		}
	}

	/*
	* @param (int) Skill's ID
	* @return (Skill)
	*/
	public static function getSkill( $id )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Skill\'s ID must be a number.' );
		if ( self::isSkillLoaded( $id ) )
			return self::$skills[$id];
		else {
			return new Skill( $id );
		}
	}

	/*
	* Return the first Skill loaded
	* @return (Skill) or null if no skill loaded
	*/
	public static function getFirst(){
		$key = array_keys(self::$skills);
		if( sizeof($key) == 0 )
			return null;
		return self::$skills[$key[0]];
	}

	/*
	   * @param (int) Skill's ID
	   * @return (bool)
	*/
	private static function isSkillLoaded( $id )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Skill\'s ID must be a number.' );
		return !empty( self::$skills[$id] );
	}

	/*
	   * Load a skill from the database
	   * @param (int) Skill's ID
	   * @return (bool)
	*/
	private static function loadSkill( $id )
	{
		if ( !is_numeric( $id ) )
			throw new Exception( 'Skill\'s ID must be a number.' );
		if ( !self::isSkillLoaded( $id ) )
			new Skill( $id );
		return true;
	}

	public static function loadAll(){
		$db = Database::getDatabase();
		$res = $db->query('SELECT `idskill` FROM `' . TBL_S . '`');
		if( $res->rowCount() > 0 ){
			foreach($res as $r)
				self::loadSkill($r['idskill']);
		}
	}

	/*
	   * Getter
	   * @param (String) Variable name
	   * @return (mixed) Value
	*/
	public function __get( $name )
	{
		if ( isSet( $this->$name ) && $name != 'skills' )
			return $this->$name;
		return null;
	}

	public function save(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		// Look if it is an update or a creation
		$update = ($this->id != 0) ? true : false;
		// table : skill
		if( $update === true ){
			$res = $db->prepare('UPDATE `' . TBL_S . '` SET `name` = :name, `description` = :description, `icon` = :icon, `gmview` = :gm_view WHERE `idskill` = :id');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':description', $this->description, PDO::PARAM_STR);
			$res->bindValue(':icon', $this->icon, PDO::PARAM_STR);
			$res->bindValue(':gm_view', $this->gmView, PDO::PARAM_INT);
			$res->bindValue(':id', $this->id, PDO::PARAM_INT);
			$res->execute();
		}else{
			$res = $db->prepare('INSERT into `' . TBL_S . '`(`name`,`description`,`icon`,`gmview`) VALUES (:name, :description, :icon, :gm_view)');
			$res->bindValue(':name', $this->name, PDO::PARAM_STR);
			$res->bindValue(':description', $this->description, PDO::PARAM_STR);
			$res->bindValue(':icon', $this->icon, PDO::PARAM_STR);
			$res->bindValue(':gm_view', (int)$this->gmView, PDO::PARAM_INT);
			$res->execute();
			$this->id = $db->lastInsertId();
		}
		$db->commit();
		return true;
	}

	public function delete(){
		$db = Database::getDatabase();
		$db->beginTransaction();
		$db->query('DELETE FROM `' . TBL_D_S . '` WHERE `idskill` = ' . $db->quote($this->id));
		$db->query('DELETE FROM `'. TBL_S . '` WHERE `idskill` = ' . $db->quote($this->id));
		$db->commit();
		return true;
	}

	public function rewind() {
		reset(self::$skills);
	}

	public function current() {
		return current(self::$skills);
	}

	public function key() {
		return key(self::$skills);
	}

	public function next() {
		next(self::$skills);
	}

	public function valid() {
		return (current(self::$skills) != false);
	}

}

?>