<?php
/*
* @author Baptiste ROUSSEL
* @version 1.0.3
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
session_start();
require_once( '../config.inc.php' );

if( empty($_SESSION['id']) ) {
	// Not loggued
	header('Location: ../index.php?login=0'); // Ask for a login
	exit();
}

try {
	if( isSet($_POST['id']) && !empty($_POST['name']) && isSet($_POST['description']) && isSet($_POST['minvalue']) && isSet($_POST['maxvalue']) && isSet($_POST['abbreviation']) ){
		$id = (int)$_POST['id'];
		if( $id == 0 || isSet($_POST['duplicate']) ){ // add a new Ability or duplicate an ability
			if( !isAllowed('ability-add') ){
				header('Location: ../panel.php?p=ability&error=4');
				exit();
			}
			$ability = new Ability();
		}elseif( isSet($_POST['submit']) ){ // update an ability
			if( !isAllowed('ability-edit') ){
				header('Location: ../panel.php?p=ability&id=' . $id . '&error=4');
				exit();
			}
			$ability = new Ability($id);
		}elseif( isSet($_POST['delete']) ){ // delete an ability
			if( !isAllowed('ability-edit') ){
				header('Location: ../panel.php?p=ability&id=' . $id . '&error=4');
				exit();
			}
			$ability = new Ability($id);
			$ability->delete();
			actionLog($lang['term']['ability'] . ' : ' . $ability->name . '(' . $id . ') ' . $lang['term']['deleted']);
			header('Location: ../panel.php?p=ability&success=1');
			exit;
		}else{// unknow state
			header('Location: ../panel.php?p=ability&error=0');
			exit();
		}
		$ability->name = $_POST['name'];
		$ability->description = $_POST['description'];
		$ability->minValue = $_POST['minvalue'];
		$ability->maxValue = $_POST['maxvalue'];
		$ability->abbreviation = $_POST['abbreviation'];
		if( $ability->save() === true ){
			$id = $ability->id;
			actionLog($lang['term']['ability'] . " : " . $ability->name . '(' . $ability->id . ') ' . $lang['term']['updated']);
			header('Location: ../panel.php?p=ability&id=' . $id . '&success=1');
			exit;
		}
	}else{
		if( empty($_POST['name']) )
			throw new InvalidArgumentException('Name required.');
	}
	if( isSet($_POST['id']) && $_POST['id'] != 0 )
		header('Location: ../panel.php?p=ability&id=' . $id . '&error=2');
	else
		header('Location: ../panel.php?p=ability&error=0');
	exit;
}catch(Exception $e){
	echo '<pre>' . $e . '</pre>';
	try {
		$db = Database::getDatabase();
		$db->rollback();
	}catch(Exception $e){}exit();
	$errorNumber = 2;
	if( $e instanceof InvalidArgumentException  ){ // required fields missing
		if( empty($data->name) )
			$_SESSION['error'][] = 'name';
		$errorNumber = 1;
	}
	if( isSet($_POST['id']) && $_POST['id'] != 0 ) // trigguered when updating
		header('Location: ../panel.php?p=ability&id='.$_POST['id'].'&error=' . $errorNumber);
	else // triggered when creating
		header('Location: ../panel.php?p=ability&error=' . $errorNumber);
	exit;
}
?>