<?php
/*
* @author Baptiste ROUSSEL
* @version 1.0.3
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
session_start();
require_once( '../config.inc.php' );

if( empty($_SESSION['id']) ) {
	// Not loggued
	header('Location: ../index.php?login=0'); // Ask for a login
	exit();
}

try {
	if( isSet($_POST['id']) && !empty($_POST['login']) && isSet($_POST['password']) && isSet($_POST['right']) ){
		if( isSet($_POST['submit']) ){
			$id = (int)$_POST['id'];
			$db = Database::getDatabase();
			if( $id == 0 ){ // create an account
				if( !isAllowed('account-add') ){
					header('Location: ../panel.php?p=account&error=4');
					exit();
				}
				$res = $db->prepare('INSERT into `' . TBL_GM_ACT . '`(`login`,`password`) VALUES(:login,:password)');
				$res->bindValue(':login',$_POST['login'],PDO::PARAM_STR);
				$res->bindValue(':password',Hash::hash($_POST['password']),PDO::PARAM_STR);
				$res->execute();
				$id = $db->lastInsertId();
			}else{ // update an account
				if( !isAllowed('account-edit') ){
					header('Location: ../panel.php?p=account&id=' . $id . '&error=4');
					exit();
				}
				if( !empty($_POST['password']) ){
					$res = $db->prepare('UPDATE `' . TBL_GM_ACT . '` SET `login` = :login, `password` = :password WHERE `idgm_account` = :id');
					$res->bindValue(':password',Hash::hash($_POST['password']),PDO::PARAM_STR);
				}else{
					$res = $db->prepare('UPDATE `' . TBL_GM_ACT . '` SET `login` = :login WHERE `idgm_account` = :id');
				}
				$res->bindValue(':login',$_POST['login'],PDO::PARAM_STR);
				$res->bindValue(':id',$id,PDO::PARAM_INT);
				$res->execute();
				$db->query('DELETE FROM `' . TBL_GM_ACC . '` WHERE `idgm_account` = ' . $db->quote($id));
			}
			$res = $db->prepare('INSERT into `' . TBL_GM_ACC . '`(`idgm_account`,`idaccess`) VALUES(:id,:idaccess)');
			$res->bindValue(':id',$id,PDO::PARAM_INT);
			foreach($_POST['right'] as $idaccess => $allowed){
				if( $allowed == '1'){
					$res->bindValue(':idaccess',$idaccess,PDO::PARAM_INT);
					$res->execute();
				}
			}
			actionLog($lang['term']['account'] . " : " . $_POST['login'] . '(' . $id . ') ' . $lang['term']['updated']);
			header('Location: ../panel.php?p=account&id=' . $id . '&success=1');
			exit;
		}elseif( isSet($_POST['delete']) && $_POST['id'] != $_SESSION['id'] ){ // delete an account
			$db = Database::getDatabase();
			$db->query('DELETE FROM `' . TBL_GM_LOG . '` WHERE `idgm_account` = ' . $db->quote($_POST['id']));
			$db->query('DELETE FROM `' . TBL_GM_ACC . '` WHERE `idgm_account` = ' . $db->quote($_POST['id']));
			$db->query('DELETE FROM `' . TBL_GM_ACT . '` WHERE `idgm_account` = ' . $db->quote($_POST['id']));
			actionLog($lang['term']['account'] . " : " . $_POST['login'] . '(' . $id . ') ' . $lang['term']['deleted']);
			header('Location: ../panel.php?p=account&success=1');
		}else
			header('Location: ../panel.php?p=account&error=0');
	}else
		throw new InvalidArgumentException('Login required.');
	if( isSet($_POST['id']) && $_POST['id'] != 0 )
		header('Location: ../panel.php?p=account&id=' . $_POST['id'] . '&error=2');
	else
		header('Location: ../panel.php?p=account&error=0');
	exit;
}catch(Exception $e){
	echo '<pre>' . $e . '</pre>';
	try {
		$db = Database::getDatabase();
		$db->rollback();exit();
	}catch(Exception $e){}
	$errorNumber = 2;
	if( $e instanceof InvalidArgumentException  ){ // required fields missing
		if( empty($data->name) )
			$_SESSION['error'][] = 'login';
		$errorNumber = 1;
	}
	if( isSet($_POST['id']) && $_POST['id'] != 0 ) // trigguered when updating
		header('Location: ../panel.php?p=account&id='.$_POST['id'].'&error=' . $errorNumber);
	else // triggered when creating
		header('Location: ../panel.php?p=account&error=' . $errorNumber);
	exit;
}
?>