<?php
/*
* @author Baptiste ROUSSEL
* @version 1.0.3
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
session_start();
require_once( '../config.inc.php' );

if( empty($_SESSION['id']) ) {
	// Not loggued
	header('Location: ../index.php?login=0'); // Ask for a login
	exit();
}

try {
	if( isSet($_POST['id']) && isSet($_POST['name']) && isSet($_POST['description']) ){
		// add a new attribute
		if( !isAllowed('fields-add') ){
			header('Location: ../panel.php?p=fields&error=4');
			exit();
		}
		if( empty($_POST['name']) )
			throw new InvalidArgumentException('Name required.');
		$db = Database::getDatabase();
		$db->query('INSERT into `attribute`(`name`,`description`) VALUES(' . $db->quote($_POST['name']) . ',' . $db->quote($_POST['description']) . ')');
	}
	if( isSet($_POST['field']) ){
		// update attributes
		if( !isAllowed('fields-edit') ){
			header('Location: ../panel.php?p=fields&error=4');
			exit();
		}
		$db = Database::getDatabase();
		$db->beginTransaction();
		// get the attributes from the database
		$res = $db->query('SELECT `idattribute`, `name`, `description` FROM `attribute` ORDER BY `idattribute`');
		if( $res->rowCount() > 0 )
			while($b = $res->fetch() )
				$attributes[$b['idattribute']] = $b;
		$logs = array();
		foreach($_POST['field'] as $id => $field){
			if( empty($field['name']) ){
				// delete the field
				$db->query('DELETE FROM `character_attribute` WHERE `idattribute` = ' . $db->quote($id));
				$db->query('DELETE FROM `attribute` WHERE `idattribute` = ' . $db->quote($id));
				$logs[] = $lang['term']['bigfield'] . '(' . $lang['term']['tinyfield'] . ') : ' . $field['name'] . '(' . $id . ') ' . $lang['term']['deleted'];
			}else{
				// compare
				if( $field['name'] != $attributes[$id]['name'] || $field['description'] != $attributes[$id]['description'] ){ // update is needed
					$db->query('UPDATE `attribute` SET `name` = ' . $db->quote($field['name']) . ', `description` = ' . $db->quote($field['description']) . ' WHERE `idattribute` = ' . $db->quote($id));
					$logs[] = $lang['term']['bigfield'] . '(' . $lang['term']['tinyfield'] . ') : ' . $field['name'] . '(' . $id . ') ' . $lang['term']['updated'];
				}
			}
		}
		foreach($logs as $log)
			actionLog($log);
		$db->commit();
	}
	header('Location: ../panel.php?p=fields&success=1');
	exit;
}catch(Exception $e){
	//echo '<pre>' . $e . '</pre>';
	try {
		$db = Database::getDatabase();
		$db->rollback();
	}catch(Exception $e){}
	$errorNumber = 2;
	if( $e instanceof InvalidArgumentException  ){ // required fields missing
		if( empty($_POST['name']) )
			$_SESSION['error'][] = 'name';
		$errorNumber = 1;
	}
	header('Location: ../panel.php?p=fields&error=' . $errorNumber);
	exit;
}
?>