<?php
/*
* @author Baptiste ROUSSEL
* @version 1.0.3
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
session_start();
require_once( '../config.inc.php' );

if( empty($_SESSION['id']) ) {
	// Not loggued
	header('Location: ../index.php?login=0'); // Ask for a login
	exit();
}

try {
	if( isSet($_POST['id']) && isSet($_POST['name']) && isSet($_POST['description']) ){
		$id = (int)$_POST['id'];
		if( $id == 0 || isSet($_POST['duplicate']) ){ // add a new classe or duplicate a classe
			if( !isAllowed('classe-add') ){
				header('Location: ../panel.php?p=classe&error=4');
				exit();
			}
			$classe = new Classe();
		}elseif( isSet($_POST['submit']) ){ // update a classe
			if( !isAllowed('classe-edit') ){
				header('Location: ../panel.php?p=classe&id=' . $id . '&error=4');
				exit();
			}
			$classe = new Classe($id);
		}elseif( isSet($_POST['delete']) ){ // Delete a classe
			if( !isAllowed('classe-edit') ){
				header('Location: ../panel.php?p=classe&id=' . $id . '&error=4');
				exit();
			}
			$classe = new Classe($id);
			if( $classe->delete() === true ){
				actionLog($lang['term']['classe'] . ' : ' . $classe->name . '(' . $id . ') ' . $lang['term']['deleted']);
				header('Location: ../panel.php?p=classe&success=1');
			}else{
				header('Location: ../panel.php?p=classe&id=' . $id . '&error=6');
			}
			exit;
		}else{// unknow state
			header('Location: ../panel.php?p=classe&error=0');
			exit();
		}
		if( empty($_POST['name']) )
			throw new InvalidArgumentException('Name required.');
		$classe->name = $_POST['name'];
		$classe->description = $_POST['description'];
		$abilities = array();
		foreach($_POST['dice'] as $id => $dice){
			$abilities[$id]['dice'] = new Dice($dice);
		}
		foreach($_POST['bonus'] as $id => $bonus){
			$abilities[$id]['bonusValue'] = $bonus;
		}
		$classe->abilities = $abilities;
		if( $classe->save() === true ){
			$id = $classe->id;
			actionLog($lang['term']['classe'] . ' : ' . $classe->name . '(' . $id . ') ' . $lang['term']['updated']);
			header('Location: ../panel.php?p=classe&id=' . $id . '&success=1');
			exit;
		}
	}
	header('Location: ../panel.php?p=classe&error=0');
	exit;
}catch(Exception $e){
	//echo '<pre>' . $e . '</pre>';
	try {
		$db = Database::getDatabase();
		$db->rollback();
	}catch(Exception $e){}
	$errorNumber = 2;
	if( $e instanceof InvalidArgumentException  ){ // required fields missing
		if( empty($_POST['name']) )
			$_SESSION['error'][] = 'name';
		$errorNumber = 1;
	}
	header('Location: ../panel.php?p=classe&error=' . $errorNumber);
	exit;
}
?>