<?php
/*
* @author Baptiste ROUSSEL
* @version 1.0.3
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
session_start();
require_once( '../config.inc.php' );

if( empty($_SESSION['id']) ) {
	// Not loggued
	header('Location: ../index.php?login=0'); // Ask for a login
	exit();
}

try {
	if( isSet($_POST['id']) && isSet($_POST['category']) && isSet($_POST['name']) && isSet($_POST['description']) && isSet($_POST['icon']) ){
		$id = (int)$_POST['id'];
		if( $id == 0 || isSet($_POST['duplicate']) ){ // add a new item or duplicate an item
			if( !isAllowed('item-add') ){
				header('Location: ../panel.php?p=item&error=4');
				exit();
			}
			$item = new Item();
		}elseif( isSet($_POST['submit']) ){ // update an item
			if( !isAllowed('item-edit') ){
				header('Location: ../panel.php?p=item&id=' . $id . '&error=4');
				exit();
			}
			$item = new Item($id);
		}elseif( isSet($_POST['delete']) ){ // Delete an item
			if( !isAllowed('item-edit') ){
				header('Location: ../panel.php?p=item&id=' . $id . '&error=4');
				exit();
			}
			$item = new Item($id);
			$item->delete();
			actionLog($lang['term']['item'] . ' : ' . $item->name . '(' . $item->id . ') ' . $lang['term']['deleted']);
			header('Location: ../panel.php?success=1');
			exit;
		}else{// unknow state
			header('Location: ../panel.php?error=0');
			exit();
		}
		if( empty($_POST['name']) )
			throw new InvalidArgumentException('Name required.');
		if( empty($_POST['category']) )
			throw new InvalidArgumentException('Category required.');
		$item->name = $_POST['name'];
		$item->categoryid = $_POST['category'];
		$item->description = $_POST['description'];
		$item->icon = $_POST['icon'];
		if( $item->save() === true ){
			$id = $item->id;
			actionLog($lang['term']['item'] . ' : ' . $item->name . '(' . $id . ') ' . $lang['term']['updated']);
			header('Location: ../panel.php?p=item&id=' . $id . '&success=1');
			exit;
		}
	}
	if( isSet($_POST['id']) && $_POST['id'] != 0 )
		header('Location: ../panel.php?p=item&id=' . $id . '&error=2');
	else
		header('Location: ../panel.php?p=add_item&error=0');
	exit;
}catch(Exception $e){
	echo '<pre>' . $e . '</pre>';
	try {
		$db = Database::getDatabase();
		$db->rollback();
	}catch(Exception $e){}
	$errorNumber = 2;
	if( $e instanceof InvalidArgumentException  ){ // required fields missing
		if( empty($_POST['name']) )
			$_SESSION['error'][] = 'name';
		if( empty($_POST['category']) )
			$_SESSION['error'][] = 'category';
		$errorNumber = 1;
	}
	if( isSet($_POST['id']) && $_POST['id'] != 0 ) // trigguered when updating
		header('Location: ../panel.php?p=item&id='.$_POST['id'].'&error=' . $errorNumber);
	else // triggered when creating
		header('Location: ../panel.php?p=add_item&error=' . $errorNumber);
	exit;
}
?>