<?php
/*
* @author Baptiste ROUSSEL
* @version 1.0.3
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
session_start();
require_once( '../config.inc.php' );

if( empty($_SESSION['id']) ) {
	// Not loggued
	header('Location: ../index.php?login=0'); // Ask for a login
	exit();
}

try {
	if( isSet($_POST['id']) && isSet($_POST['name']) ){
		// add a new item category
		if( !isAllowed('item_category-add') ){
			header('Location: ../panel.php?p=item_category&error=4');
			exit();
		}
		if( empty($_POST['name']) )
			throw new InvalidArgumentException('Name required.');
		$db = Database::getDatabase();
		$db->query('INSERT into `' . TBL_I_CAT . '`(`name`) VALUES(' . $db->quote($_POST['name']) . ')');
		$id = $db->lastInsertId();
		actionLog($lang['term']['item_category'] . ' : ' . $field . '(' . $id . ') ' . $lang['term']['updated']);
	}
	if( isSet($_POST['field']) ){
		// update item categories
		if( !isAllowed('item_category-edit') ){
			header('Location: ../panel.php?p=item_category&error=4');
			exit();
		}
		$db = Database::getDatabase();
		$db->beginTransaction();
		// get the bigfields from the database
		$categories = array();
		$res = $db->query('SELECT `iditemcategory`, `name` FROM `item_category` ORDER BY `iditemcategory`');
		if( $res->rowCount() > 0 )
			while($b = $res->fetch() )
				$categories[$b['iditemcategory']] = $b['name'];
		$logs = array();
		foreach($_POST['field'] as $id => $field){
			if( empty($field) ){
				// delete the category
				$db->query('DELETE FROM `' . TBL_D_I . '` WHERE `iditem` IN (SELECT `iditem` FROM `item` WHERE `iditemcategory` = ' . $db->quote($id) . ')');
				$db->query('DELETE FROM `' . TBL_I . '` WHERE `iditemcategory` = ' . $db->quote($id));
				$db->query('DELETE FROM `' . TBL_I_CAT . '` WHERE `iditemcategory` = ' . $db->quote($id));
				$logs[] = $lang['term']['item_category'] . ' : ' . $field . '(' . $id . ') ' . $lang['term']['deleted'];
			}else{
				// compare
				if( isSet($categories[$id]) && $field != $categories[$id] ){ // update is needed
					$db->query('UPDATE `item_category` SET `name` = ' . $db->quote($field) . ' WHERE `iditemcategory` = ' . $db->quote($id));
					$logs[] = $lang['term']['item_category'] . ' : ' . $field . '(' . $id . ') ' . $lang['term']['updated'];
				}
			}
		}
		$db->commit();
		foreach($logs as $log)
			actionLog($log);
	}
	header('Location: ../panel.php?p=item_category&success=1');
	exit;
}catch(Exception $e){
	//echo '<pre>' . $e . '</pre>';
	try {
		$db = Database::getDatabase();
		$db->rollback();
	}catch(Exception $e){}
	$errorNumber = 2;
	if( $e instanceof InvalidArgumentException  ){ // required fields missing
		if( empty($_POST['name']) )
			$_SESSION['error'][] = 'name';
		$errorNumber = 1;
	}
	header('Location: ../panel.php?p=item_category&error=' . $errorNumber);
	exit;
}
?>