<?php
/*
* @author Baptiste ROUSSEL
* @version 1.0.3
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
session_start();
require_once( '../config.inc.php' );

if( empty($_SESSION['id']) ) {
	// Not loggued
	header('Location: ../index.php?login=0'); // Ask for a login
	exit();
}

if( !isSet($_POST['id']) ){
	// Data's ID is missing
	header('Location: ../index.php?cat=panel&error=0');
	exit();
}

try {
	if( $_POST['id'] == 0 ){
		if( !isAllowed('sheet-add') ){
			header('Location: ../panel.php?p=sheet_search&error=4');
			exit();
		}
		$data = new Data();
	}else{
		if( !isAllowed('ability-edit') ){
			header('Location: ../panel.php?p=sheet&id=' . $_POST['id'] . '&error=4');
			exit();
		}
		$data = new Data($_POST['id']);
	}

	if( isSet($_POST['submit']) ){ // save a sheet
		if( empty($_POST['data_name']) )
			throw new InvalidArgumentException ('Name required.');
		if( empty($_POST['type']) )
			throw new InvalidArgumentException ('Type required.');
		if( empty($_POST['classe']) )
			throw new InvalidArgumentException ('Classe required.');

		if( isSet($_POST['data_name']) && $data->name != $_POST['data_name'] )
			$data->name = $_POST['data_name'];
		if( isSet($_POST['type']) && $data->type->id != $_POST['type'] ){
			$data->type = new Type($_POST['type']);
		}
		if( isSet($_POST['classe']) && $data->classe->id != $_POST['classe']){
			$data->classe = new Classe($_POST['classe']);
		}
		if( isSet($_POST['data_image']) && $data->image != $_POST['data_image'])
			$data->image = $_POST['data_image'];
		if( isSet($_POST['attr']) ){
			foreach($_POST['attr'] as $id => $value){
				if( !isSet($data->attributes[$id]['value']) || $data->attributes[$id]['value'] != $value )
					$data->attributes[$id]['value'] = $value;
			}
		}
		if( isSet($_POST['fields']) ){
			foreach($_POST['fields'] as $id => $value){
				if( !isSet($data->fields[$id]['value']) || $data->fields[$id]['value'] != $value )
					$data->fields[$id]['value'] = $value;
				else
					$data->fields[$id]['value'] = null;
			}
		}
		if( isSet($_POST['skills']) ){
			$skills = array();
			foreach($_POST['skills'] as $id){
				$skills[$id] = '';
			}
			$data->skills = $skills;
		}
		if( isSet($_POST['ability']['type']) && isSet($_POST['ability']['classe']) ){
			$ability = array();
			foreach($_POST['ability']['type'] as $id => $value){
				$ability[$id]['rolledValueType'] = $value;
			}
			foreach($_POST['ability']['classe'] as $id => $value){
				$ability[$id]['rolledValueClasse'] = $value;
			}
			$data->abilities = $ability;
		}
		if( isSet($_POST['inventory']) ){
			$inventory = array();
			foreach($_POST['inventory'] as $id){
				if( isSet($inventory[$id]) )
					$inventory[$id]['quantity']++;
				else
					$inventory[$id]['quantity'] = 1;
			}
			$data->items = $inventory;
		}

		$success = $data->save();
		if( $success === true ){ 
			actionLog($lang['term']['data'] . ' : ' . $data->name . '(' . $data->id . ') ' . $lang['term']['updated']);
			header('Location: ../panel.php?p=sheet&id=' . $data->id . '&success=1');
		}elseif( $id == 0 )
			header('Location: ../panel.php?p=sheet&id=' . $data->id . '&error=0');
		else
			header('Location: ../panel.php?p=sheet&error=2');

	}elseif( isSet($_POST['delete']) ){ // delete a sheet
		$success = $data->delete();
		if( $success === true ){
			actionLog($lang['term']['data'] . ' : ' . $data->name . '(' . $data->id . ') ' . $lang['term']['deleted']);
			header('Location: ../panel.php?success=1');
		}else
			header('Location: ../panel.php?p=sheet&id=' . $_POST['id'] . '&error=2');
	}
	
}catch(Exception $e){
	//echo '<pre>' . $e . '</pre>';
	try {
		$db = Database::getDatabase();
		$db->rollback();
	}catch(Exception $e){}
	$errorNumber = 2;
	if( $e instanceof InvalidArgumentException  ){ // required fields missing
		if( empty($data->name) )
			$_SESSION['error'][] = 'name';
		if( $data->type === null )
			$_SESSION['error'][] = 'type';
		if( $data->classe === null )
			$_SESSION['error'][] = 'classe';
		$errorNumber = 1;
	}
	if( $data->id != 0 ) // trigguered when updating
		header('Location: ../panel.php?p=sheet&id=' . $data->id . '&error=' . $errorNumber);
	else // triggered when creating
		header('Location: ../panel.php?p=add_sheet&error=' . $errorNumber);

}
?>