<?php
/*
* @author Baptiste ROUSSEL
* @version 1.0.3
* @license MPL 2.0
* This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. 
* If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
session_start();
require_once( '../config.inc.php' );

if( empty($_SESSION['id']) ) {
	// Not loggued
	header('Location: ../index.php?login=0'); // Ask for a login
	exit();
}

try {
	if( isSet($_POST['id']) && isSet($_POST['name']) && isSet($_POST['description']) && isSet($_POST['icon']) ){
		$id = (int)$_POST['id'];
		if( $id == 0 || isSet($_POST['duplicate']) ){ // add a new skill or duplicate a skill
			if( !isAllowed('skill-add') ){
				header('Location: ../panel.php?p=skill&error=4');
				exit();
			}
			$skill = new Skill();
		}elseif( isSet($_POST['submit']) ){ // update a skill
			if( !isAllowed('skill-edit') ){
				header('Location: ../panel.php?p=skill&id=' . $id . '&error=4');
				exit();
			}
			$skill = new Skill($id);
		}elseif( isSet($_POST['delete']) ){ // delete a skill
			if( !isAllowed('skill-edit') ){
				header('Location: ../panel.php?p=skill&id=' . $id . '&error=4');
				exit();
			}
			$skill = new Skill($id);
			$skill->delete();
			actionLog($lang['term']['skill'] . ' : ' . $skill->name . '(' . $id . ') ' . $lang['term']['deleted']);
			header('Location: ../panel.php?success=1');
			exit;
		}else{ // unknow state
			header('Location: ../panel.php?p=add_skill&error=0');
			exit();
		}
		// Manage datas
		if( empty($_POST['name']) )
			throw new InvalidArgumentException('Name required.');
		$skill->name = $_POST['name'];
		$skill->description = $_POST['description'];
		$skill->icon = $_POST['icon'];
		if( $skill->save() === true ){
			$id = $skill->id;
			actionLog($lang['term']['skill'] . ' : ' . $skill->name . '(' . $id . ') ' . $lang['term']['updated']);
			header('Location: ../panel.php?p=skill&id=' . $id . '&success=1');
			exit;
		}
	}
	if( isSet($_POST['id']) && $_POST['id'] != 0 )
		header('Location: ../panel.php?p=skill&id=' . $id . '&error=2');
	else
		header('Location: ../panel.php?p=add_skill&error=0');
	exit();
}catch(Exception $e){
	//echo '<pre>' . $e . '</pre>';
	try {
		$db = Database::getDatabase();
		$db->rollback();
	}catch(Exception $e){}
	$errorNumber = 2;
	if( $e instanceof InvalidArgumentException  ){ // required fields missing
		if( empty($_POST['name']) )
			$_SESSION['error'][] = 'name';
		$errorNumber = 1;
	}
	if( isSet($_POST['id']) && $_POST['id'] != 0 ) // trigguered when updating
		header('Location: ../panel.php?p=skill&id='.$_POST['id'].'&error=' . $errorNumber);
	else // triggered when creating
		header('Location: ../panel.php?p=add_skill&error=' . $errorNumber);
	exit;
}
?>